import { world, system, TicksPerSecond } from "@minecraft/server"
import "./items";

function isEntityOnScreen(player, entity, screenValue) {
    //Higher screenValue = Closer crosshair has to be to the entity for it to be considered on screen
    let direction = {
        x: entity.location.x - player.location.x,
        y: entity.location.y - player.location.y,
        z: entity.location.z - player.location.z
    };
    let length = Math.sqrt(direction.x ** 2 + direction.y ** 2 + direction.z ** 2);
    direction.x /= length;
    direction.y /= length;
    direction.z /= length;
    let viewDirection = player.getViewDirection();
    let dotProduct = direction.x * viewDirection.x + direction.y * viewDirection.y + direction.z * viewDirection.z;
    const value = screenValue;
    if (dotProduct > value) {
        let currentPos = { ...player.location };
        let stepSize = 0.5;
        for (let i = 0; i < length / stepSize; i++) {
            currentPos.x += direction.x * stepSize;
            currentPos.y += direction.y * stepSize;
            currentPos.z += direction.z * stepSize;
            let blockLocation = {
                x: Math.floor(currentPos.x),
                y: Math.floor(currentPos.y),
                z: Math.floor(currentPos.z)
            };
            const block = player.dimension.getBlock({ x: blockLocation.x, y: blockLocation.y + 1, z: blockLocation.z });

            //Blocks that will be filtered out of the raycast (The script will "see through" them)
            if (block && (block.typeId != "minecraft:air") && (block.typeId != "minecraft:snow_layer") && (block.typeId != "minecraft:short_grass") && (block.typeId != "minecraft:tall_grass")) {
                return false;
            }
        }
        return true;
    }
    else
        return false;
}

function circleFunction() {
    const circleListOverworld = world.getDimension("overworld").getEntities({ type: 'minigames:circle', });
    const circleListNether = world.getDimension("nether").getEntities({ type: 'minigames:circle', });
    const circleListEnd = world.getDimension("the_end").getEntities({ type: 'minigames:circle', });
    const circleList = circleListOverworld.concat(circleListNether, circleListEnd);
    for (const circle of circleList) {
        for (const player of world.getPlayers({ excludeGameModes: ['spectator'], })) {
            if (isEntityOnScreen(player, circle, 0.9995) && player.hasTag("minigames:in_minigame")) {
                if (player.getDynamicProperty("score_count") <= 78) {
                    player.setDynamicProperty("score_count", (player.getDynamicProperty("score_count") + 3));
                }
                else if (player.getDynamicProperty("score_count") == 79) {
                    player.setDynamicProperty("score_count", 80);
                }
                if (player.getDynamicProperty("score_count") <= 80 && player.getDynamicProperty("score_count") > 72) {
                    player.runCommand("title @s actionbar ||||||||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 72 && player.getDynamicProperty("score_count") > 64) {
                    player.runCommand("title @s actionbar ||||||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 64 && player.getDynamicProperty("score_count") > 56) {
                    player.runCommand("title @s actionbar ||||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 56 && player.getDynamicProperty("score_count") > 48) {
                    player.runCommand("title @s actionbar ||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 48 && player.getDynamicProperty("score_count") > 40) {
                    player.runCommand("title @s actionbar ||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 40 && player.getDynamicProperty("score_count") > 32) {
                    player.runCommand("title @s actionbar ||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 32 && player.getDynamicProperty("score_count") > 24) {
                    player.runCommand("title @s actionbar ||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 24 && player.getDynamicProperty("score_count") > 16) {
                    player.runCommand("title @s actionbar ||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 16 && player.getDynamicProperty("score_count") > 8) {
                    player.runCommand("title @s actionbar ||||")
                }
                else if (player.getDynamicProperty("score_count") <= 8 && player.getDynamicProperty("score_count") > 0) {
                    player.runCommand("title @s actionbar ||")
                }
            }
            else if (!isEntityOnScreen(player, circle, 0.9995) && player.hasTag("minigames:in_minigame")) {
                if (player.getDynamicProperty("score_count") > 0) {
                    player.setDynamicProperty("score_count", (player.getDynamicProperty("score_count") - 1));
                }
                else { //lose state
                    player.runCommand("function end_minigames");
                    player.runCommand("tp @e [type=minigames:pande,c=1] ^ ^0.5 ^0.5")
                    player.runCommand("playsound deathplayerpande @s ~ ~ ~ 0.5 1");
                    player.runCommand("event entity @e[type=minigames:circle,c=2] minigames:death");
                    player.runCommand("tag @s add minigames:game_lost");
                    player.runCommand("event entity @e[type=minigames:pande,c=1] minigames:you_lose");
                    player.runCommand("kill @s");
                }
                if (player.getDynamicProperty("score_count") <= 80 && player.getDynamicProperty("score_count") > 72) {
                    player.runCommand("title @s actionbar §4||||||||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 72 && player.getDynamicProperty("score_count") > 64) {
                    player.runCommand("title @s actionbar §4||||||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 64 && player.getDynamicProperty("score_count") > 56) {
                    player.runCommand("title @s actionbar §4||||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 56 && player.getDynamicProperty("score_count") > 48) {
                    player.runCommand("title @s actionbar §4||||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 48 && player.getDynamicProperty("score_count") > 40) {
                    player.runCommand("title @s actionbar §4||||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 40 && player.getDynamicProperty("score_count") > 32) {
                    player.runCommand("title @s actionbar §4||||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 32 && player.getDynamicProperty("score_count") > 24) {
                    player.runCommand("title @s actionbar §4||||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 24 && player.getDynamicProperty("score_count") > 16) {
                    player.runCommand("title @s actionbar §4||||||")
                }
                else if (player.getDynamicProperty("score_count") <= 16 && player.getDynamicProperty("score_count") > 8) {
                    player.runCommand("title @s actionbar §4||||")
                }
                else if (player.getDynamicProperty("score_count") <= 8 && player.getDynamicProperty("score_count") > 0) {
                    player.runCommand("title @s actionbar §4||")
                }
            }
        }
    }
}

system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id == "minigames:start_minigame") {
        for (const player of world.getPlayers({ tags: ['minigames:minigame_player'], })) {
            player.runCommand("tag @s remove minigames:minigame_player");
            player.runCommand("tag @s add minigames:in_minigame");
            player.runCommand("inputpermission set @s movement disabled");
            player.runCommand("inputpermission set @s camera enabled");
            player.runCommand("particle minigames:minigame_spawn ~ ~ ~");
            if (!world.gameRules.sendCommandFeedback) {
                player.runCommand("tp @s ~ ~ ~ ~ 0");
            }
            else {
                world.gameRules.sendCommandFeedback = false;
                player.runCommand("tp @s ~ ~ ~ ~ 0");
                world.gameRules.sendCommandFeedback = true;
            }
            player.runCommand("summon minigames:circle ^ ^ ^12");
            player.runCommand("summon minigames:pande ^ ^ ^12 ~ ~ minigames:test");
            player.runCommand("tp @e[type=minigames:pande,c=1] @e[type=minigames:circle,c=1]");
            const gameTimeEntity = player.dimension.spawnEntity("minigames:game_time_entity", player.location);
            player.runCommand("tag @s add minigames:" + gameTimeEntity.id);
        }
    }
    if (data.id == "minigames:start_minigame_43_seconds") {
        for (const player of world.getPlayers({ tags: ['minigames:minigame_player'], })) {
            player.runCommand("tag @s remove minigames:minigame_player");
            player.runCommand("tag @s add minigames:in_minigame");
            player.runCommand("inputpermission set @s movement disabled");
            player.runCommand("inputpermission set @s camera enabled");
            player.runCommand("particle minigames:minigame_spawn ~ ~ ~");
            if (!world.gameRules.sendCommandFeedback) {
                player.runCommand("tp @s ~ ~ ~ ~ 0");
            }
            else {
                world.gameRules.sendCommandFeedback = false;
                player.runCommand("tp @s ~ ~ ~ ~ 0");
                world.gameRules.sendCommandFeedback = true;
            }
            player.runCommand("summon minigames:circle ^ ^ ^12");
            player.runCommand("summon minigames:pande ^ ^ ^12 ~ ~ minigames:fourthreeseconds");
            player.runCommand("tp @e[type=minigames:pande,c=1] @e[type=minigames:circle,c=1]");
            const gameTimeEntity = player.dimension.spawnEntity("minigames:game_time_entity<minigames:43_seconds>", player.location);
            player.runCommand("tag @s add minigames:43_seconds");
            player.runCommand("tag @s add minigames:" + gameTimeEntity.id);
        }
    }
    else if (data.id == "minigames:end_minigames") {
        for (const player of world.getPlayers({ tags: ['minigames:in_minigame'], })) {
            player.runCommand("tag @s remove minigames:in_minigame");
            player.runCommand("inputpermission set @s movement enabled");
            player.runCommand("event entity @e[type=minigames:rotation_entity] minigames:despawn");
        }
    }
    else if (data.id == "minigames:player_face_rotation_entity") {
        for (const player of world.getPlayers({ tags: ['minigames:in_minigame'], })) {
            if (!world.gameRules.sendCommandFeedback) {
                player.runCommand("tp @s ~ ~ ~ facing @e [type=minigames:rotation_entity,c=1]");
            }
            else {
                world.gameRules.sendCommandFeedback = false;
                player.runCommand("tp @s ~ ~ ~ facing @e [type=minigames:rotation_entity,c=1]");
                world.gameRules.sendCommandFeedback = true;
            }
        }
    }
    else if (data.id == "minigames:display_game_time") {
        world.sendMessage("gameTime: " + data.sourceEntity.getProperty("minigames:game_time"));
    }
});

function applyTimer() {
    for (const player of world.getPlayers()) {
        if (player.getDynamicProperty("score_count") == null) {
            player.setDynamicProperty("score_count", 80);
            //world.sendMessage(player.name + "'s Score Count: " + player.getDynamicProperty("score_count")); //debug
        }
    }
}

function gameTimeSystem() {
    for (const player of world.getPlayers()) {
        if (player.hasTag("minigames:in_minigame")) {
            const allGameTimesOverworld = world.getDimension("overworld").getEntities({ type: 'minigames:game_time_entity', });
            const allGameTimesNether = world.getDimension("nether").getEntities({ type: 'minigames:game_time_entity', });
            const allGameTimesEnd = world.getDimension("the_end").getEntities({ type: 'minigames:game_time_entity', });
            const allGameTimes = allGameTimesOverworld.concat(allGameTimesNether, allGameTimesEnd);
            for (const gameTimeEntity of allGameTimes) {
                if (player.hasTag("minigames:" + gameTimeEntity.id)) {
                    let gameTime = gameTimeEntity.getProperty("minigames:game_time");
                    if (gameTime == 46) { //win state
                        player.runCommand("tag @s remove minigames:" + gameTimeEntity.id);
                        player.runCommand("function end_minigames");
                        player.runCommand("summon minigames:lootwin");
                        player.runCommand("event entity @e[type=minigames:pande,c=2] minigames:die_normal");
                        player.runCommand("event entity @e[type=minigames:circle,c=2] minigames:death")
                    }
                    else if (gameTime == 43 && player.hasTag("minigames:43_seconds")) { //win state 43 second version
                        player.runCommand("tag @s remove minigames:" + gameTimeEntity.id);
                        player.runCommand("tag @s remove minigames:43_seconds");
                        player.runCommand("function end_minigames");
                        player.runCommand("summon minigames:lootwine");
                        player.runCommand("event entity @e[type=minigames:circle,c=2] minigames:death");
                        player.runCommand("event entity @e[type=minigames:pande,c=2] minigames:die_normal")
                    }
                    else if (player.hasTag("minigames:game_lost")) { //remove tags and game_time_entity when losing
                        player.runCommand("tag @s remove minigames:game_lost");
                        player.runCommand("tag @s remove minigames:" + gameTimeEntity.id);
                        gameTimeEntity.runCommand("event entity @s minigames:despawn")
                    }
                }
            }
        }
    }
}

system.runInterval(() => {
    world.getDimension("overworld").runCommandAsync('function particle_portal');
    world.getDimension("nether").runCommandAsync('function particle_portal');
    world.getDimension("the_end").runCommandAsync('function particle_portal');
}, 30);

system.runInterval(() => {
    world.getDimension("overworld").runCommandAsync('function particle_portal_two');
    world.getDimension("nether").runCommandAsync('function particle_portal_two');
    world.getDimension("the_end").runCommandAsync('function particle_portal_two');
}, 50);

system.runInterval(circleFunction, 1);

system.runInterval(applyTimer, 1);

system.runInterval(gameTimeSystem, 1);

world.afterEvents.entitySpawn.subscribe((data) => {
    if (data.entity.typeId == "minigames:pande") {
        data.entity.runCommandAsync('particle minigames:minigame_spawn ~ ~ ~');
    }
});